<?php
/**
* PHP5 Wrapper classes for the Xapian library (http://www.xapian.org)
* Version: 0.1 
* Date   : 04-April-2006
* 
* Copyright  2006 Paul Dixon (paul@elphin.com)
* blog : http://blog.dixo.net
* email: lordelph@gmail.com
* 
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
* 
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
* 
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

/**
* USAGE
*
* You should find these wrappers "natural" to use if you read the Xapian C++
* API documentation. The only difference is that all classes are prefixed with
* "Xapian" so the C++ Xapian::Document class becomes XapianDocument
*
* The only real difference is that the various Iterator classes support 
* the PHP5 Iterator interface, allowing you to use them in a foreach loop,
* for example
*
* $termi= $db->allterms_begin();
* foreach($termi as $term)
* {
*    echo "$term | ";
* }
*
* All of the classes will accept an Xapian resource in their constructor
* which will wrap that resource, for example, to open an in-memory database
* you could use
*
* $db = new XapianWritableDatabase(inmemory_open());
*
* Alternatively, pass the document construction parameters to construct a 
* new resource, e.g.
* 
* $database = new XapianDatabase($dbfile);
*
* Inheritance is supported, for example you can call XapianDatabase methods on
* a XapianWritableDatabase object
*
* PERFORMANCE
*
* The xapian_php5_smoketest.php is an adaptation of the original smoketest.php
* which comes with the xapian-bindings library. The PHP5 version of the smoketest
* runs around 2-3 times slower than the raw API calls. 
*
* The smoketest is almost entire made up of Xapian calls, so if Xapian calls are
* your bottleneck, then these classes will slow you down more
*
* However, in most common uses, particularly querying, API calls are unlikely to
* be the main bottleneck and using php5 iterators makes for some easy to read code.
* 
*
* IMPLEMENTATION NOTES
*
* Implementation of this set of wrapper classes has been greatly simplified
* by the use of a base class to handle the method calls via the php5 __call
* override. 
*
* Generally, the only methods explicitly defined are those which return
* iterator classes
*/




/**
* XapianClassWrapper provides automatic method calling for wrapped classes,
* which simplifies the development of the wrapper
*/
abstract class XapianClassWrapper {
    
   	/**
    * The wrapped Xapian resource
    */
    public $ptr=null;
   	
   	/**
   	* static method which given a result will tell you if it
   	* is a Xapian resource and optionally verify the expected
   	* type
   	*/
    static public function isXapianResource($res, $required_type=null) {
    	$ok=is_resource($res);
		if ($ok)
		{
			//exapian resources look like this
			//_p_Xapian__Database
			$type=explode('_', get_resource_type($res));
			
			$ok=(count($type)>=5) &&
				($type[2]=='Xapian') &&
				( ($required_type==null) || ($required_type==$type[4]));
		}
		
		return $ok;
    }
   
   	/**
   	* given Xapian resource, find class name
   	*/
   static public function getClassnameFromResource($res) {
		$type=explode('_', get_resource_type($res));
		return $type[4];	
	}

	/**
   	* Smart constructor will wrap a Xapian resource if a
   	* suitable one is passed, otherwise will call the relevant
   	* new(..) method passing all parameters along with it
   	*/
   	public function __construct($ptr=null) {
		//we can do a variety of clever things here - firstly, if the 
		//param is a resource and it's type matches the class we're
		//wrapping, we can use it as our ptr
		
		$create_resource=true;
		if (is_resource($ptr))
		{
			$class=get_class($this);
			while (strpos($class,'Xapian')!==0)
				$class=get_parent_class($class);
			
			$xapian_class=substr($class,6);
			
			if (XapianClassWrapper::isXapianResource($ptr, $xapian_class))
			{
				$create_resource=false;
				$this->ptr=$ptr;
			}
		}
		
		if ($create_resource)
		{
			//we weren't passed a Xapian resource, which means we probably
			//want to call new, so we call it and pass all the parameters we
			//were passed...
			
			$args=func_get_args();
			$this->ptr=call_user_method_array ('new', $this, $args);
	
		}
	}
   
   	public function coerce_param(&$value, $index)
   	{
   		if (is_object($value) && is_a($value, 'XapianClassWrapper'))
			$value=$value->ptr;	
   	}
   	
   	
   	
   	
	/**
   	* This handles most Xapian method calls by figuring out which 
   	* function should be called and amending the parameters to suit it
   	*/
   	private function __call($m, $a) {
		//figure out function name
		
		//find the XapianXYZ parent class - from this we determine
		//which type of Xapian object is wrapped. To do this we
		//go up the inheritance chain until we find a class starting
		//with Xapian
		$class=get_class($this);
		while (strpos($class,'Xapian')!==0)
			$class=get_parent_class($class);
		
		$xapian_class=substr($class,6);
		
		//now we have a xapian class, we can see if thed require method
		//is mapped
		$func=($m!='new')?($xapian_class.'_'.$m):($m.'_'.$xapian_class);
		while (!function_exists($func))
		{
			//not available, but maybe in a parent class?
			$class=get_parent_class($class);
			if (!empty($class) && ($class!='XapianClassWrapper'))
			{
				$xapian_class=substr($class,6);	
				$func=($m!='new')?($xapian_class.'_'.$m):($m.'_'.$xapian_class);
			}
			else
			{
				//we've reached the top of the chain, no joy...
				break;	
			}
		
		}
		
		if (function_exists($func)) {
			
			//no point passing instances of this class to xapian, but
			//it's nice to do from from the outside, so let's coerce those
			//into the right type
			array_walk_recursive($a, array($this, 'coerce_param'));
			
			//pass ptr as first param if function isn't a
			//"new" call
			if ($m!='new') 
				array_unshift($a, $this->ptr);
			
			$rvalue=call_user_func_array($func, $a);
			if (is_resource($rvalue) && ($m!='new'))
			{
				//wrap in class?
				$xapianclass=XapianClassWrapper::getClassnameFromResource($rvalue);
				$wrapperclass='Xapian'.$xapianclass;
				if (class_exists($wrapperclass))
				{
					//create a new instance of the appropriate wrapper - 
					//note that we pass a second parameter so that the class
					//can find it's parent - this is used by iterator wrappers
					//to provide php5 iterators 
					$rvalue=new $wrapperclass($rvalue);
					
				}
					
			}

			return $rvalue;

		}
		else {
			throw new Exception("Xapian $func not found"); 
		}
   }	
}

/**
* BM25 weighting scheme
*/
class XapianBM25Weight extends XapianWeight {
	/* The following methods are handled by the base class
	new XapianBM25Weight ($k1_, $k2_, $k3_, $b_, $min_normlen_)
	
	clone()
	name()
	serialise()
	unserialise($str)
	get_sumpart($wdf, $len)
	get_maxpart()
	get_sumextra($len)
	get_maxextra90
	get_sumpart_needs_doclength()
    */
}


/**
* Boolean weighting scheme (everything gets 0)
*/
class XapianBoolWeight extends XapianWeight {
	/* The following methods are handled by the base class
	new XapianBoolWeight()
	clone()
	name()
	serialise()
	unserialise($str)
	get_sumpart($wdf, $len)
	get_maxpart()
	get_sumextra($len)
	get_maxextra90
	get_sumpart_needs_doclength()
    */
}


/**
* This class is used to access a database, or a group of databases
*/
class XapianDatabase extends XapianClassWrapper {
	/* The following methods are handled by the base class
	* new XapianDatabase($path)
	* add_database(XapianDatabase $db)
	* allterms_end()
	* get_avlength()
	* get_collection_freq($tname)
	* get_description()
	* get_doccount()
	* get_doclength($docid)
	* get_document($docid)
	* get_lastdocid()
	* get_termfreq($tname)
	* keep_alive()
	* positionlist_end($docid,$tname)
	* postlist_end($tname)
	* reopen()
	* term_exists($name)
	* termlist_end($docid)
    */
	
	/**
	* An iterator which runs across all terms in the database. 
	*
	* This returns a PHP5TermIterator to provide a
	* class which also implements the PHP5 Iterator interface,
	* allowing the iterator to be used in a foreach loop - both
	* the key and value are the term.
	*
	* Example of use:
	*
	* $termi= $db->allterms_begin();
	* foreach($termi as $term)
	* {
	*    echo "$term | ";
	* }
	*/
	function allterms_begin()
	{
		$begin=Database_allterms_begin($this->ptr);
    	$end=Database_allterms_end($this->ptr);
    	return new PHP5TermIterator($begin,$end);
	}

    /**
	* An iterator for position list for a given term in a given document.
	*
	* This returns a PHP5PositionIterator to provide a
	* class which also implements the PHP5 Iterator interface,
	* allowing the iterator to be used in a foreach loop. The
	* key and value are both the numeric position value
	*
	* Example of use:
	*
	* $posi= $db->positionlist_begin($docid,$tname);
	* foreach($posi as $pos)
	* {
	*    echo "position $pos\n";
	* }
	*/
	function positionlist_begin($docid,$tname)
    {
    	$begin=Database_positionlist_begin($this->ptr, $docid, $tname);
    	$end=Database_positionlist_end($this->ptr, $docid, $tname);
    	return new PHP5PositionIterator($begin,$end);
    }
    
    /**
	* An iterator for the postlist for a given term. 
	*
	* This returns a PHP5PostingIterator to provide a
	* class which also implements the PHP5 Iterator interface,
	* allowing the iterator to be used in a foreach loop. The
	* key is the docid and the value is the wdf
	*
	* Example of use:
	*
	* $posti= $db->postlist_begin($tname);
	* foreach($posti as $docid => $wdf)
	* {
	*    echo "docid $docid\n";
	*    echo "wdf $wdf\n";
	*    $length = $posti->get_doclength();
	*
	*    $posi = $posti->positionlist_begin();
	*    foreach($posi as $pos)
	*    {
	*       echo "position $pos\n";
	*    }
	* }
	*/
	function postlist_begin($tname)
    {
    	$begin=Database_postlist_begin($this->ptr, $tname);
    	$end=Database_postlist_end($this->ptr, $tname);
    	return new PHP5PostingIterator($begin,$end);
    }
    
    
    /**
	* Iterator for the terms in a given document. 
	*
	* This returns a PHP5TermIterator to provide a
	* class which also implements the PHP5 Iterator interface,
	* allowing the iterator to be used in a foreach loop - both
	* the key and value are the term.
	*
	* Example of use:
	*
	* $termi= $db->termlist_begin($docid);
	* foreach($termi as $term)
	* {
	*    echo "$term | ";
	* }
	*/
	function termlist_begin($docid)
    {
    	$begin=Database_termlist_begin($this->ptr, $docid);
    	$end=Database_termlist_end($this->ptr, $docid);
    	return new PHP5TermIterator($begin,$end);
    }
    
    
}


/**
* A document in the database - holds data, values, terms, and postings
*/
class XapianDocument extends XapianClassWrapper
{
   /**
    * The following methods are handled by the base class
	* new XapianDocument()
	* add_posting($tname, $tpos, $wdfinc=1)
	* add_term($tname, $wdfinc=1);
	* add_value($valueno, $value)
	* clear_terms()
	* clear_values()
	* get_data()
	* get_description()
	* get_value($valueno)
	* remove_posting($tname, $tpos, $wdfdec=1)
	* remove_term($tname)
	* remove_value($valueno)
	* set_data($data)
	* termlist_count()
	* termlist_end()
	* values_count()
	* values_end()
    */
       
	/**
	* Iterator for the terms in this document. 
	*
	* This returns a PHP5TermIterator to provide a
	* class which also implements the PHP5 Iterator interface,
	* allowing the iterator to be used in a foreach loop - both
	* the key and value are the term.
	*
	* Example of use:
	*
	* $termi= $doc->termlist_begin();
	* foreach($termi as $term)
	* {
	*    echo "$term | ";
	* }
	*/
	function termlist_begin()
	{
		$begin=Document_termlist_begin($this->ptr);
    	$end=Document_termlist_end($this->ptr);
    	return new PHP5TermIterator($begin,$end);
	}
	
	/**
	* Iterator for the values in this document. 
	*
	* This returns a PHP5ValueIterator to provide a
	* class which also implements the PHP5 Iterator interface,
	* allowing the iterator to be used in a foreach loop - the key
	* is the value number and the value is the, er, value :)
	*
	* Example of use:
	*
	* $vali= $doc->values_begin();
	* foreach($vali as $valueno=>$value)
	* {
	*    echo "$valueno = $value\n";
	* }
	*/
	function values_begin()
	{
		$begin=Document_valuelist_begin($this->ptr);
    	$end=Document_valuelist_end($this->ptr);
    	return new PHP5ValueIterator($begin,$end);
		
	}
	
}


/**
* This class provides an interface to the information retrieval system for the purpose of searching
*/
class XapianEnquire extends XapianClassWrapper {
	
	/**
	* The following methods are handled by the base class
	* new XapianEnquire($db)
	* get_description()
	* get_eset($maxitems, $omrset, $edecider)
	* get_matching_terms ??
	* get_matching_terms_end($it)
	* get_mset($first, $maxitems, $omrset, $mdecider)
	* get_query()
	* register_match_decider($name, $mdecider)
	* set_bias($bias_weight, $bias_halflife)
	* set_collapse_key($valueno)
	* set_cutoff($percent_cutoff, $weight_cutoff=0)
	* set_docid_order($order)
	* set_query($query, $qlen)
	* set_sort_by_relevance()
	* set_sort_by_value($sort_key, $ascending=true)
	* set_sort_by_value_then_relevance($sort_key, $ascending=true)
	* set_weighting_scheme($weight)
	*/ 	
	
	/**
	* Get terms which match a given document, by document id, 
	* or get terms which match a given document, by match set item. 
	*
	* This returns a PHP5TermIterator to provide a
	* class which also implements the PHP5 Iterator interface,
	* allowing the iterator to be used in a foreach loop - both
	* the key and value are the term.
	*
	* Example of use:
	*
	* $termi= $enquire->get_matching_terms_begin($docid);
	* foreach($termi as $term)
	* {
	*    echo "$term | ";
	* }
	*/
	function get_matching_terms_begin($it_or_docid)
	{
		$begin=Enquire_get_matching_terms_begin($this->ptr, $it_or_docid);
    	$end=Enquire_get_matching_terms_end($this->ptr, $it_or_docid);
    	return new PHP5TermIterator($begin,$end);
	}
	
}


/**
* Class representing an ordered set of expand terms (an ESet)
*/
class XapianESet extends XapianClassWrapper {
	
	/* 
	* base class implements these methods
	* back()
	* empty()
	* end()
	* get_description()
	* get_ebound()
	* is_empty()
	* size()
    */
    
    /**
	* Iterator for the terms in this E-Set. 
	*
	* This returns a PHP5ESetIterator to provide a
	* class which also implements the PHP5 Iterator interface,
	* allowing the iterator to be used in a foreach loop. The
	* key is the term and the value is the weight
	*
	* Example of use:
	*
	* $eseti= $eset->begin();
	* foreach($eseti as $term=>$weight)
	* {
	*    echo "$term has weight $weight\n";
	* }
	*/
	function begin()
    {
    	$begin=ESet_begin($this->ptr);
    	$end=ESet_end($this->ptr);
    	return new PHP5ESetIterator($begin,$end);
    }
}


/**
* Iterate through terms in the ESet

   The base class implements these member functions
   equals($iterator)
   get_description()
   get_termname()
   get_weight()
   next()
   prev()
    
*/
class XapianESetIterator extends XapianClassWrapper {
}

/**
* An iterator pointing to items in a list of positions
*
* This extends XapianPositionIterator to provide a
* class which also implements the PHP5 Iterator interface,
* allowing the iterator to be used in a foreach loop. The
* key is the term and the value is the weight
*
* Example of use:
*
* $eseti= $eset->begin();
* foreach($eseti as $term=>$weight)
* {
*    echo "$term has weight $weight\n";
* }
*/
class PHP5ESetIterator extends XapianESetIterator implements Iterator
{
	protected $ptrEnd=null;
	protected $used=false;
	
	public function __construct($begin, $end) {
		
		assert(XapianClassWrapper::isXapianResource($begin,'ESetIterator'));
		assert(XapianClassWrapper::isXapianResource($end,'ESetIterator'));
		
		parent::__construct($begin);
		$this->ptrEnd = $end;
	}

	/**
	* Implementation of Iterator::rewind
	* Allow use of an instance of this class within a foreach loop
	*/
	public function rewind() {
		if ($this->used)
			throw new Exception("PHP5ESetIterator cannot be rewound"); 
			
	}

	
	/**
	* Implementation of Iterator::next
	* Allow use of an instance of this class within a foreach loop
	*/
	public function next() {
		$this->used=true;
		return ESetIterator_next($this->ptr);
	}
	
	/**
	* Implementation of Iterator::valid
	* Allow use of an instance of this class within a foreach loop
	*/
	public function valid() {
		return !ESetIterator_equals($this->ptr, $this->ptrEnd);
	}
	
	/**
	* Implementation of Iterator::key
	* Allow use of an instance of this class within a foreach loop
	*/
	public function key() {
		return ESetIterator_get_termname($this->ptr);
	}

	/**
	* Implementation of Iterator::current
	* Allow use of an instance of this class within a foreach loop
	*/
	public function current() {
		return ESetIterator_get_weight($this->ptr);
    }	
	
}




/**
* A match set (MSet)
*/
class XapianMSet extends XapianClassWrapper
{
    /*These methods are provided by the XapianClassWrapper
	back()
	convert_to_percent($weight_or_iterator)
	empty()
	end()
	fetch(...)
	get_description()
	get_document($i)
	get_document_id() ?
	get_document_percentage() ?
	get_firstitem()
	get_hit() ?
	get_matches_estimated()
	get_matches_lower_bound()
	get_matches_upper_bound()
	get_max_attained()
	get_max_possible()
	get_termfreq($term)
	get_termweight($term)
	is_empty()
	size()
	*/

	/**
	* An iterator pointing to items in an MSet
	*
	* This returns a PHP5MSetIterator to provide a
	* class which also implements the PHP5 Iterator interface,
	* allowing the iterator to be used in a foreach loop. The key
	* the docid and the value is a XapianDocument object
	*
	* Example of use:
	*
	* $iterator=$mset->begin();
	* foreach($iterator as $docid => $doc){
	*     $percent=$iterator->get_percent();
	* }
	*/
	function begin()
	{
		$begin=MSet_begin($this->ptr);
    	$end=MSet_end($this->ptr);
    	return new PHP5MSetIterator($begin,$end);
    }
}


/**
* An iterator pointing to items in an MSet
*/
class XapianMSetIterator extends XapianClassWrapper {
	
	//these methods are handled by XapianClassWrapper
	//public function get_document()
	//public function get_percent()
	//public function get_rank()
	//public function get_weight()
	//public function get_collapse_count()
	//public function get_description()
	//public function get_docid()
	//public function prev()
	
}

/**
* An iterator pointing to items in an MSet
*
* This wraps the Xapian::MSetIterator methods to provide a
* class which also implements the PHP5 Iterator interface,
* allowing the iterator to be used in a foreach loop. The key
* the docid and the value is a XapianDocument object
*
* Example of use:
*
* $iterator=$mset->begin();
* foreach($iterator as $docid => $doc){
*     $percent=$iterator->get_percent();
* }
*/
class PHP5MSetIterator extends XapianMSetIterator implements Iterator
{
	protected $ptrEnd=null;
	protected $used=false;
	
	public function __construct($begin, $end) {
		
		assert(XapianClassWrapper::isXapianResource($begin,'MSetIterator'));
		assert(XapianClassWrapper::isXapianResource($end,'MSetIterator'));
		
		parent::__construct($begin);
		$this->ptrEnd = $end;
	}
	
	

	/**
	* Implementation of Iterator::rewind
	* Allow use of an instance of this class within a foreach loop
	*/
	public function rewind() {
		if ($this->used)
			throw new Exception("PHP5MSetIterator cannot be rewound");
	}

	
	/**
	* Implementation of Iterator::next
	* Allow use of an instance of this class within a foreach loop
	*/
	public function next() {
		$this->used=true;
		return MSetIterator_next($this->ptr);
	}
	
	/**
	* Implementation of Iterator::valid
	* Allow use of an instance of this class within a foreach loop
	*/
	public function valid() {
		return !MSetIterator_equals($this->ptr, $this->ptrEnd);
	}

	/**
	* Implementation of Iterator::key
	* Allow use of an instance of this class within a foreach loop
	*/
	public function key() {
		return MSetIterator_get_docid($this->ptr);
	}

	/**
	* Implementation of Iterator::current
	* Allow use of an instance of this class within a foreach loop
	*/
	public function current() {
		return new XapianDocument(MSetIterator_get_document($this->ptr));
    }	
	
}


/**
* An iterator pointing to items in a list of positions
*/
class XapianPositionIterator extends XapianClassWrapper {
}

/**
* An iterator pointing to items in a list of positions
*
* This extends XapianPositionIterator to provide a
* class which also implements the PHP5 Iterator interface,
* allowing the iterator to be used in a foreach loop. The
* key and value are both the numeric position value
*
* Example of use:
*
* $posi= $db->positionlist_begin($docid,$tname);
* foreach($posi as $pos)
* {
*    echo "position $pos\n";
* }
*/
class PHP5PositionIterator extends XapianPositionIterator implements Iterator
{
	protected $ptrEnd=null;
	protected $used=false;
	
	public function __construct($begin, $end) {
		
		assert(XapianClassWrapper::isXapianResource($begin,'PositionIterator'));
		assert(XapianClassWrapper::isXapianResource($end,'PositionIterator'));
		
		parent::__construct($begin);
		$this->ptrEnd = $end;
	}

	/**
	* Implementation of Iterator::rewind
	* Allow use of an instance of this class within a foreach loop
	*/
	public function rewind() {
		if ($this->used)
			throw new Exception("PHP5PositionIterator cannot be rewound");
	}

	
	/**
	* Implementation of Iterator::next
	* Allow use of an instance of this class within a foreach loop
	*/
	public function next() {
		$this->used=true;
		return PositionIterator_next($this->ptr);
	}
	
	/**
	* Implementation of Iterator::valid
	* Allow use of an instance of this class within a foreach loop
	*/
	public function valid() {
		return !PositionIterator_equals($this->ptr, $this->ptrEnd);
	}
	
	/**
	* Implementation of Iterator::key
	* Allow use of an instance of this class within a foreach loop
	*/
	public function key() {
		return PositionIterator_get_termpos($this->ptr);
	}

	/**
	* Implementation of Iterator::current
	* Allow use of an instance of this class within a foreach loop
	*/
	public function current() {
		return PositionIterator_get_termpos($this->ptr);
    }	
	
}



/**
* An iterator pointing to items in a list of postings
*/
class XapianPostingIterator extends XapianClassWrapper {
}

/**
* An iterator pointing to items in a list of postings
*
* This extends XapianPostingIterator to provide a
* class which also implements the PHP5 Iterator interface,
* allowing the iterator to be used in a foreach loop. The
* key is the docid and the value is the wdf
*
* Example of use:
*
* $posti= $db->postlist_begin($tname);
* foreach($posti as $docid => $wdf)
* {
*    echo "docid $docid\n";
*    echo "wdf $wdf\n";
*    $length = $posti->get_doclength();
*
*    $posi = $posti->positionlist_begin();
*    foreach($posi as $pos)
*    {
*       echo "position $pos\n";
*    }
* }
*/
class PHP5PostingIterator extends XapianPostingIterator implements Iterator
{
	protected $ptrEnd=null;
	protected $used=false;
	
	public function __construct($begin, $end) {
		
		assert(XapianClassWrapper::isXapianResource($begin,'PostingIterator'));
		assert(XapianClassWrapper::isXapianResource($end,'PostingIterator'));
		
		parent::__construct($begin);
		$this->ptrEnd = $end;
	}

    /**
	* An iterator pointing to items in a list of positions
	*
	* This returns a PHP5PositionIterator to provide a
	* class which also implements the PHP5 Iterator interface,
	* allowing the iterator to be used in a foreach loop. The
	* key and value are both the numeric position value
	*
	* Example of use:
	*
	* $posi= $posti->positionlist_begin();
	* foreach($posi as $pos)
	* {
	*    echo "position $pos\n";
	* }
	*/
	function positionlist_begin()
    {
    	$begin=PostingIterator_positionlist_begin($this->ptr);
    	$end=PostingIterator_positionlist_end($this->ptr);
    	return new PHP5PositionIterator($begin,$end);
    }
    
	/**
	* Implementation of Iterator::rewind
	* Allow use of an instance of this class within a foreach loop
	*/
	public function rewind() {
		if ($this->used)
			throw new Exception("PHP5PostingIterator cannot be rewound");
	}

	
	/**
	* Implementation of Iterator::next
	* Allow use of an instance of this class within a foreach loop
	*/
	public function next() {
		$this->used=true;
		return PostingIterator_next($this->ptr);
	}
	
	/**
	* Implementation of Iterator::valid
	* Allow use of an instance of this class within a foreach loop
	*/
	public function valid() {
		return !PostingIterator_equals($this->ptr, $this->ptrEnd);
	}
	
	/**
	* Implementation of Iterator::key
	* Allow use of an instance of this class within a foreach loop
	*/
	public function key() {
		return PostingIterator_get_docid($this->ptr);
	}

	/**
	* Implementation of Iterator::current
	* Allow use of an instance of this class within a foreach loop
	*/
	public function current() {
		return PostingIterator_get_wdf($this->ptr);
    }	
	
}

/**
* Class representing a query
*/
class XapianQuery extends XapianClassWrapper {
	/* base class implements these methods 
	new XapianQuery(...)
	empty()
	get_description()
	get_length()
	get_terms_end()
	is_empty()
	*/
	
	
	/**
	* Get terms in the query
	*
	* This returns a PHP5TermIterator to provide a
	* class which also implements the PHP5 Iterator interface,
	* allowing the iterator to be used in a foreach loop - both
	* the key and value are the term.
	*
	* Example of use:
	*
	* $termi= $query->get_terms_begin();
	* foreach($termi as $term)
	* {
	*    echo "$term | ";
	* }
	*/
	function get_terms_begin()
	{
		$begin=Query_get_terms_begin($this->ptr);
    	$end=Query_get_terms_end($this->ptr);
    	return new PHP5TermIterator($begin,$end);
	}
        
}


/**
* Build a Xapian::Query object from a user query string
*/
class XapianQueryParser extends XapianClassWrapper {

	/* base class implements these methods
	add_boolean_prefix($field, $prefix)
	add_prefix($field, $prefix)
	get_default_op()
	get_description()
	parse_query($query_string, $flags=FLAG_PHRASE|FLAG_BOOLEAN|FLAG_LOVEHATE)
	set_database($db)
	set_default_op($op)
	set_stemmer($stemmer)
	set_stemming_strategy($strategy)
	set_stopper($stopper)
	stoplist_end()
	unstem_end($term)
	*/
	
	/**
	* Iterate over terms omitted from the query as stopwords. 
	*
	* This returns a PHP5TermIterator to provide a
	* class which also implements the PHP5 Iterator interface,
	* allowing the iterator to be used in a foreach loop - both
	* the key and value are the term.
	*
	* Example of use:
	*
	* $termi= $queryparser->stoplist_begin();
	* foreach($termi as $term)
	* {
	*    echo "$term | ";
	* }
	*/
	function stoplist_begin()
	{
		$begin=QueryParser_stoplist_begin($this->ptr);
    	$end=QueryParser_stoplist_end($this->ptr);
    	return new PHP5TermIterator($begin,$end);
	}
	
	/**
	* Iterate over unstemmed forms of the given (stemmed) term used in the query. 
	*
	* This returns a PHP5TermIterator to provide a
	* class which also implements the PHP5 Iterator interface,
	* allowing the iterator to be used in a foreach loop - both
	* the key and value are the term.
	*
	* Example of use:
	*
	* $termi= $queryparser->unstem_begin($term);
	* foreach($termi as $term)
	* {
	*    echo "$term | ";
	* }
	*/
	function unstem_begin($term)
	{
		$begin=QueryParser_unstem_begin($this->ptr, $term);
    	$end=QueryParser_unstem_end($this->ptr, $term);
    	return new PHP5TermIterator($begin,$end);
	}
}

/**
* A relevance set (R-Set)
*/
class XapianRSet extends XapianClassWrapper {
	
	/*
	base class implements these methods
	
	add_document($doc)
	contains($docid)
	empty()
	get_description()
	is_empty()
	remove_document($docid)
	size()
    */
    
}


/**
* Simple implementation of Stopper class - this will suit most users
*/
class XapianSimpleStopper extends XapianStopper {
	/*
	base class implements these methods
	
	add($word)
	apply()
	get_description()
	*/
}


/**
* Class representing a stemming algorithm
*/
class XapianStem extends XapianClassWrapper {
	/*
	base class implements these methods
	apply()
	get_available_languages()
	get_description()
	stem_word($word)
	*/
}


/**
* Base class for stop-word decision functor
*/
class XapianStopper extends XapianClassWrapper {
	/*
	base class implements these methods
	apply()
	get_description()
	*/
    	
}


/**
* An iterator pointing to items in a list of terms
* Note this exactly wraps the provided functionality, but
* a more useful derived class PHP5TermIterator is available
*/
class XapianTermIterator extends  XapianClassWrapper 
{
}

/**
* An iterator pointing to items in a list of terms
*
* This extends XapianTermIterator to provide a
* class which also implements the PHP5 Iterator interface,
* allowing the iterator to be used in a foreach loop - both
* the key and value are the term.
*
* Example of use:
*
* $termi= $doc->termlist_begin();
* foreach($termi as $term)
* {
*    echo "$term | ";
* }
*/
class PHP5TermIterator extends XapianTermIterator implements Iterator
{
	protected $ptrEnd=null;
	protected $used=false;
	
	public function __construct($begin, $end) {
		
		assert(XapianClassWrapper::isXapianResource($begin,'TermIterator'));
		assert(XapianClassWrapper::isXapianResource($end,'TermIterator'));
		
		parent::__construct($begin);
		$this->ptrEnd = $end;
	}

	/**
	* Implementation of Iterator::rewind
	* Allow use of an instance of this class within a foreach loop
	*/
	public function rewind() {
		if ($this->used)
			throw new Exception("PHP5TermIterator cannot be rewound");
	}

	
	/**
	* Implementation of Iterator::next
	* Allow use of an instance of this class within a foreach loop
	*/
	public function next() {
		$this->used=true;
		return TermIterator_next($this->ptr);
	}
	
	/**
	* Implementation of Iterator::valid
	* Allow use of an instance of this class within a foreach loop
	*/
	public function valid() {
		return !TermIterator_equals($this->ptr, $this->ptrEnd);
	}
	
	/**
	* Implementation of Iterator::key
	* Allow use of an instance of this class within a foreach loop
	*/
	public function key() {
		return TermIterator_get_term($this->ptr);
	}

	/**
	* Implementation of Iterator::current
	* Allow use of an instance of this class within a foreach loop
	*/
	public function current() {
		return TermIterator_get_term($this->ptr);
    }	
	
}


/**
* Traditional probabilistic weighting scheme (as used by Muscat 3.6)
*/
class XapianTradWeight extends XapianWeight {
	/* base class implements these methods
	clone()
	get_maxextra()
	get_maxpart()
	get_sumextra($len)
	get_sumpart($wdf, $len)
	get_sumpart_needs_doclength()
	name()
	serialise()
	unserialise($str)
	*/
}


/**
* An iterator pointing to values associated with a document
*/
class XapianValueIterator extends XapianClassWrapper {
}

/**
* An iterator pointing to items in a list of terms
*
* This extends XapianTermIterator to provide a
* class which also implements the PHP5 Iterator interface,
* allowing the iterator to be used in a foreach loop - the key
* is the value number and the value is the, er, value :)
*
* Example of use:
*
* $vali= $doc->values_begin();
* foreach($vali as $valueno=>$value)
* {
*    echo "$valueno = $value\n";
* }
*/
class PHP5ValueIterator extends XapianValueIterator implements Iterator
{
	protected $ptrEnd=null;
	protected $used=false;
	
	public function __construct($begin, $end) {
		
		assert(XapianClassWrapper::isXapianResource($begin,'ValueIterator'));
		assert(XapianClassWrapper::isXapianResource($end,'ValueIterator'));
		
		parent::__construct($begin);
		$this->ptrEnd = $end;
	}

	/**
	* Implementation of Iterator::rewind
	* Allow use of an instance of this class within a foreach loop
	*/
	public function rewind() {
		if ($this->used)
			throw new Exception("PHP5TermIterator cannot be rewound");
	}

	
	/**
	* Implementation of Iterator::next
	* Allow use of an instance of this class within a foreach loop
	*/
	public function next() {
		$this->used=true;
		return ValueIterator_next($this->ptr);
	}
	
	/**
	* Implementation of Iterator::valid
	* Allow use of an instance of this class within a foreach loop
	*/
	public function valid() {
		return !ValueIterator_equals($this->ptr, $this->ptrEnd);
	}
	
	/**
	* Implementation of Iterator::key
	* Allow use of an instance of this class within a foreach loop
	*/
	public function key() {
		return ValueIterator_get_valueno($this->ptr);
	}

	/**
	* Implementation of Iterator::current
	* Allow use of an instance of this class within a foreach loop
	*/
	public function current() {
		return ValueIterator_get_value($this->ptr);
    }	
	
}


/**
* Abstract base class for weighting schemes
*/
class XapianWeight extends XapianClassWrapper {
	/* base class implements these
	
	get_maxextra()
	get_maxpart()
	get_sumextra($len)
	get_sumpart($wdf, $len)
	get_sumpart_needs_doclength()
	name()
	serialise()
	unserialise($str)
	*/	
}

/**
* This class provides read/write access to a database
*/
class XapianWritableDatabase extends XapianDatabase {
	/* base class implements these
	
	add_document($doc)
	begin_transaction()
	cancel_transaction()
	commit_transaction()
	delete_document($docid)
	flush()
	get_description()
	replace_document($term, $doc)
	*/
}




?>
